#version 300 es
precision lowp float;
uniform mat4 uMatrix;
uniform float uTime;
uniform float uCircleRandom;
uniform float uSunPosOffsetY;
uniform vec2 uSunPos;
uniform vec2 uResolution;
uniform float uLineAlpha;
in vec3 aPosition;
in vec2 aUv;

out vec2 vUv;
out vec4 light;
out float circleNumber;
out float randSeed;
out vec2 sunPos;
out mat4 worldSpaceMat;

#define LINE_SPEED 0.5
#define LINE_COLOR vec4(1.0, 0.8078, 0.3922, 1.0)
#define PI 3.1415926

float getRandom(float value, float seed) {
    return fract(sin(value * 999.9) * seed);
}

float perFromVal(float val, float from, float to) {
    return (val - from) / (to - from);
}

void main() {

    //鼠标控制太阳位置的逻辑
    sunPos = uSunPos.xy;
    sunPos.x += uSunPosOffsetY * sunPos.x;
    sunPos.y += uSunPosOffsetY;
    sunPos.x *= uResolution.x / uResolution.y;
    float time = sin(uTime * 0.3333 + 5.) * 0.05; //用于影响光斑的大小，以及距离太阳的距离
    float lightTime = (sin(uTime * LINE_SPEED * 4.) + 1.) * 0.5; //影响弱光线1动画
    float lightTime2 = (sin((uTime * LINE_SPEED + PI * 0.3333) * 4.) + 1.) * 0.5; //影响弱光线2动画
    float lightTime3 = (sin((uTime * LINE_SPEED + PI * 0.6667) * 4.) + 1.) * 0.5; // 影响弱光线3动画
    circleNumber = floor(mix(4., 8., getRandom(17., 7.8 / 100000. * 67.9))); //随机光斑数量，在4-8之间
    randSeed = 7.8 / 100000. * 939.7;
    light = vec4(lightTime, lightTime2, lightTime3, time);
    light.rgb *= LINE_COLOR.rgb * (uLineAlpha / 320.);

    // 构造透视矩阵
    // float aspect = uResolution.y / uResolution.x;
    float fov = 140.0;
    float aspect = 1.0; // the aspect ratio has already been applied to uv
    float zNear = 0.1;
    float zFar = 100.0;
    float zRange = zNear - zFar;
    float tanHalfFOV = tan(radians(fov / 2.0));
    mat4 perspective = mat4(
    1.0 / (tanHalfFOV * aspect), 0.0, 0.0, 0.0,
    0.0, 1.0 / tanHalfFOV, 0.0, 0.0,
    0.0, 0.0, (-zNear - zFar) / zRange, 2.0 * zFar * zNear / zRange,
    0.0, 0.0, 1.0, 0.0
    );

    float fx = perFromVal(sunPos.x, -1.0, 1.0);
    float rx = radians(20.0);
    float ry = radians(mix(15.0, -15.0, fx));
    mat4 rotateX = mat4(
    1.0, 0.0, 0.0, 0.0,
    0.0, cos(rx), -sin(rx), 0.0,
    0.0, sin(rx), cos(rx), 0.0,
    0.0, 0.0, 0.0, 1.0
    );
    mat4 rotateY = mat4(
    cos(ry), 0.0, sin(ry), 0.0,
    0.0, 1.0, 0.0, 0.0,
    -sin(ry), 0.0, cos(ry), 0.0,
    0.0, 0.0, 0.0, 1.0
    );
    mat4 rotate = rotateY * rotateX;
    mat4 translate = mat4(
    1.0, 0.0, 0.0, 0.0,
    0.0, 1.0, 0.0, 0.0,
    0.0, 0.0, 1.0, 0.0,
    0.0, 0.0, 3.0, 1.0
    );
    // 构造三维面片
    worldSpaceMat = perspective * translate * rotate;

    vUv = aUv - 0.5; //将uv坐标系的原点转移到屏幕中心
    vUv.x *= uResolution.x / uResolution.y; //固定宽高比
    gl_Position = uMatrix * vec4(aPosition, 1.0);
}