#version 300 es
precision highp float;

in vec2 vUv;
out vec4 fragColor;

precision highp float;

#define PI 3.14159265

// 在光线路径上 采样的次数
#define STEP_COUNT 20
// layout(local_size_x = 16, local_size_y = 16, local_size_z = 1) in;
// 指定输出图像的格式和绑定点
// layout(rgba8,binding = 0) highp restrict writeonly uniform image2D out_image;

// 散射光的Rayleigh系数
uniform vec3 ScatterRayleigh;
// Rayleigh散射层的密度
uniform float HDensityRayleigh;
// Mie散射系数
uniform float ScatterMie;
// Mie散射的不对称性因子
uniform float AsymmetryMie;
// Mie吸收系数
uniform float AbsorbMie;
// Mie吸收层的密度
uniform float HDensityMie;
// 臭氧吸收系数
uniform vec3 AbsorbOzone;
// 臭氧层中心高度
uniform float OzoneCenterHeight;
// 臭氧厚度
uniform float OzoneThickness;
// 行星半径
uniform float PlanetRadius;
// 大气半径
uniform float AtmosphereRadius;

// uniform vec2 destRect;


// 计算在高度h处的总吸收系数。
vec3 getSigmaT(float h)
{
    // 计算Rayleigh散射对总吸收系数的贡献
    // 高度 `h` 处Rayleigh散射对总吸收系数的贡献。
    vec3 rayleigh = ScatterRayleigh * vec3(exp(-h / HDensityRayleigh));
    // 计算mie散射对总吸收系数的贡献
    float mie = (ScatterMie + AbsorbMie) * exp(-h / HDensityMie);
    // 计算臭氧吸收对总吸收系数的贡献
    vec3 ozone = AbsorbOzone *vec3(max(0.0f, 1. - 0.5 * abs(float(h) - OzoneCenterHeight) / OzoneThickness));
    return rayleigh + vec3(mie) + ozone;
}

// 计算光线与圆的交点
bool findClosestIntersectionWithCircle(vec2 o, vec2 d, float R, out float t){
    // 光线方向向量的长度的平方
    float A = dot(d, d);
    // 计算光线起点到圆心的向量与光线方向向量的点击的两倍
    float B = 2. * dot(o, d);
    // 计算光线起点到圆心的向量的长度的平方减去圆的半径的平方
    float C = dot(o, o) - R*R;

    float delta = B*B - 4.*A*C;
    // 如果判别式小于零，说明光线与圆没有交点，返回 `false`。
    if (delta < 0.){
        return false;
    }
    // 计算光线与圆的交点到光线起点的距离
    t = (-B + (C<=0.? sqrt(delta):-sqrt(delta)))/(2.*A);
    return (C<=0.)||(B<=0.);
}

/**
vUv.x 决定高度
vUv.y 决定角度
*/
void main(){
    // ivec2 size = ivec2(1,1);//imageSize(out_image);
    vec2 size = vec2(1.);//uResolution;

    // if (int(vUv.x) >= size.x  || int(vUv.y) >= size.y)
    // return;

    // 计算当前线程的纬度角度。   asin：传入正弦值 返回一个角度  theta: [-π/2,π/2]
    // float theta = asin(mix(-1.0, 1.0, (float(vUv.y)+ 0.5)/float(size.y)));
    float theta = asin((vUv.y * 2. - 1.));

    // 计算当前线程的高度
    // float h = mix(0., AtmosphereRadius - PlanetRadius, (float(vUv.x) + 0.5)/float(size.x));
    float h = mix(0., AtmosphereRadius - PlanetRadius, vUv.x);

    // 计算光线的起点
    vec2 start = vec2(0., PlanetRadius + h);
    // 计算光线的方向
    vec2 direction = vec2(cos(theta), sin(theta));
    // 光线与圆的交点距离
    float distance = 0.;
    // 计算光线与行星表面和大气层的交点
    if (!findClosestIntersectionWithCircle(start, direction, PlanetRadius, distance)){
        findClosestIntersectionWithCircle(start, direction, AtmosphereRadius, distance);
    }
    // 计算光线的终点
    vec2 end = start + vec2(distance)*direction;

    vec3 sum;
    // 在光线路径上采样，计算总吸收系数
    for (int i = 0;i < STEP_COUNT; ++i)
    {
        vec2 pi = mix(start, end, vec2((float(i) + 0.5)/float(STEP_COUNT)));
        float hi = length(pi) - PlanetRadius;
        // 计算在高度h处的总吸收系数。
        vec3 sigma = getSigmaT(hi);
        sum += sigma;
    }
    // 计算光线的透射率
    vec3 result = exp(-sum * vec3(distance/float(STEP_COUNT)));
    // 将结果保存
    // imageStore(out_image, ivec2(vUv.xy), vec4(result, 1.));

    fragColor = vec4(result, 1.);
    // fragColor = vec4(vUv, 0., 1.);
}



