#version 300 es
precision highp float;
in vec2 vUv;
out vec4 fragColor;

uniform float uTime;
uniform vec2 uResolution;
uniform float uOpacity;

uniform vec3 uCamPos;
uniform float uFov;
uniform vec3 uPos;
uniform float uRadius;

uniform float uBrightness0;
uniform float uBrightness1;

uniform float uGap;
uniform float uVisRange;

#define PI 3.1415926

float getRandom(float value, float seed) {
    return fract(sin(value * 999.9) * seed);
}

vec3 clamp3(vec3 color, float min, float max) {
    return clamp(color, min, max);
}

vec4 circleTex(sampler2D tex, vec2 st) {
    return texture(tex, vec2(length(st * 2.0 - 1.0), 0.5));
}

const float kFarClip = 1000.0;

// vec3 vSunDir = normalize(vec3(-0.5, -1.5, 1.0));

float Checker(vec2 uv) {
    return step(fract((floor(uv.x) + floor(uv.y)) * 0.5), 0.25);
}

float RainbowShadow(vec3 p) {
    float f = -p.x * 1.1 + p.y - 1.5 - p.z;
    float fSpread = 0.05;
    return clamp(f * fSpread + 0.1, 0.0, 1.0);
}

float smin(float a, float b, float k) {
    float h = clamp(0.5 + 0.5 * (b - a) / k, 0.0, 1.0);
    return mix(b, a, h) - k * h * (1.0 - h);
}

float smax(float a, float b, float k) { return -smin(-a, -b, k); }

float Quadratic(float x, float a, float b) { return a * x * x + b * x; }

vec3 GetSpectrum(float x, vec3 b, vec3 peak, vec3 range) {
    vec3 vTemp = 1.0 - abs((peak - x) * range);
    vTemp = clamp(vTemp, 0.0, 1.0);
    vec3 vTemp2 = vTemp * vTemp;
    return (3.0 * vTemp2 - 2.0 * vTemp * vTemp2) * b;
}

vec4 GetRainbowRGBA(float theta) {
    vec4 vResult = vec4(0.0);

    // 第一个彩虹的峰值和范围
    vec3 vPeak0 = vec3(180.0 - 137.7, 180.0 - (137.7 + 139.6) * 0.5, 180.0 - 139.6) + vec3((uRadius - 0.5) * 180.0);
    vec3 vRange0 = 1.0 / vec3(vPeak0.b - vPeak0.r);
    vec3 vBrightness0 = vec3(uBrightness0);

    vResult.xyz += GetSpectrum(theta, vBrightness0, vPeak0, vRange0);

    // 第二个彩虹的峰值和范围
    vec3 vPeak1 = vec3(180.0 - 129.5, 180.0 - (129.5 + 126.1) * 0.5, 180.0 - 126.1) + vec3((uGap - 0.5) * 180.0) + vec3((uRadius - 0.5) * 180.0);
    vec3 vRange1 = 1.0 / vec3(vPeak1.b - vPeak1.r);
    vec3 vBrightness1 = vec3(uBrightness1);
    //vec3 vBrightness1 = vec3(0.0);

    vResult.xyz += GetSpectrum(theta, vBrightness1, vPeak1, vRange1);

    // 计算透明度
    float aFactor = max(smoothstep(vPeak0.r + 1.0, vPeak0.b - 1.0, theta), smoothstep(vPeak1.r - 1.0, vPeak1.b + 1.0, theta) * 0.2);
    //float aFactor = smoothstep(vPeak0.r + 1.0, vPeak0.b - 1.0, theta);
    vResult.a = 1.0 - aFactor;

    return vResult;
}

vec4 ApplyAtmosphere(vec3 vRayOrigin, vec3 vRayDir, vec4 vResult) {
    // vec3 vCentre = vec3(0.0, 0.0, uVisRange * 5.0);
    vec3 vCentre = vec3(0.0);

    vec3 vOffset = vCentre - vRayOrigin;
    float d = dot(vRayDir, vOffset);
    vec3 vClosest = vRayOrigin + vRayDir * d;
    float l = length(vClosest - vCentre);

    float r = uVisRange;

    float hcl = sqrt(r * r - l * l);

    float t0 = d - hcl;

    float dt = 10.;
    float f = (r - l) / r;
    f = f * f * f;
    float fOpticalDepth = dt * f * 0.1;
    float fAmount = 1.0 - exp2(-fOpticalDepth);

    float fCameraShadow = RainbowShadow(vRayOrigin) * 0.75 + 0.25;

    float fRainbowAmount = clamp(fAmount * fCameraShadow * 1.0, 0.0, 1.0);

    vec3 vSunDir = normalize(vec3(-0.5, -1.5, 1.0));
    vSunDir = normalize(uPos);

    float fDp = dot(vSunDir, vRayDir);
    vec4 vRainbow = GetRainbowRGBA(degrees(acos(fDp)) + 0.5);
    vResult.xyz += vRainbow.xyz * fRainbowAmount;

    return vResult;
}

vec4 getWeatherEffect(vec2 uv) {

    vec2 st = (uv - 0.5) * vec2(uResolution.x / uResolution.y, 1.0) * 2.0;

    vec3 forward = normalize(-uCamPos);
    vec3 right = normalize(cross(vec3(0.0, 1.0, 0.0), forward));
    vec3 up = cross(forward, right);

    vec3 rayOrigin = uCamPos;
    vec3 rayDir = normalize(st.x * right * uFov + st.y * up * uFov + forward);

    vec3 color = ApplyAtmosphere(rayOrigin, rayDir, vec4(0.)).xyz;

    return vec4(color, 1.0);
}

void main() {
    vec2 uv = vec2(1.0-vUv.x,vUv.y-0.16);
    vec4 effect = getWeatherEffect(uv);
    effect.a *= uOpacity;
    effect.xyz *= 0.6;//0.6亮度系数
    fragColor = effect;
}
