#version 300 es
precision lowp float;

#define PI 3.141592653589793
#define ONE_OVER_PI 0.3183098861837907

struct DirectionalLight {
    vec3 direction;
    vec3 color;
    float intensity;
};
struct ColorConfig {
    bool fromLight;
    float picker;
    float multi;
    float intensity;
    float whitePer;
};

in vec2 vUv;
in vec2 vPosition;
in float vLandMode;
out vec4 fragColor;

uniform sampler2D uTex;

uniform float normalScale;

uniform float opacity;

uniform vec3 directionalLights0;
uniform vec3 directionalLights1;
uniform vec3 directionalLights2;
uniform ColorConfig configs[5];

uniform float uSmooth;
uniform float uProgress;
uniform float uReverse;

uniform sampler2D uBgLightTex;
uniform sampler2D uCloudNoiseTex;
uniform float uStartEdge;
uniform float uEndEdge;

uniform float uFogDensity;

uniform vec2 SunDirection;


uniform vec3 FrustumA;
uniform vec3 FrustumC;

//uniform int isBehindSun;
uniform float uSunIntensity;

uniform float whitePer;
uniform float cloudDepth;

uniform float uInnerIntensity;

vec2 texCoord2uv(vec2 _texCoord)
{
    vec2 texCoord = _texCoord;
    texCoord.y = 1. - texCoord.y;
    float u = _texCoord.x;
    vec3 dir = normalize(mix(FrustumA, FrustumC, texCoord.y));
    float theta = asin(dir.y);
    float v = 0.5 + 0.5 * sign(theta) * sqrt(abs(theta) / (PI / 2.));
    return vec2(u, v);
}

vec3 bgEnviromentLight(float probe, float yOffset){

    vec2 st = vec2(SunDirection.y, probe);
    st = texCoord2uv(st);
    st.y += yOffset;
    st.y = (st.y * 256.0 - 115.0) / 141.0;
    return texture(uBgLightTex, st).rgb;
}

vec3 bgEnviromentLight(float probe){
    return bgEnviromentLight(probe, 0.);
}

float perFromVal(float val, float from, float to) {
    return (val - from) / (to - from);
}

float matchAlphaToAlpha(float curAlpha, float tarAlpha) {
    return (tarAlpha - curAlpha) / (1. - curAlpha);
}

float calcDepthAlpha(float depth){
    float per = (1. + uSmooth) - clamp(uProgress, 0., 1.);
    return clamp(perFromVal(depth, per - uSmooth, per), 0., 1.);
}


float depthCuttingAlpha(float alpha, float depthAlpha){
    float smoothAlpha = clamp(uProgress, 0., 2.);
    smoothAlpha = 1.- clamp(smoothAlpha-1., 0.,1.);
    smoothAlpha *= clamp(uProgress*3., 0., 1.);
    bool reverse = uReverse == 1.;
    alpha *= reverse ? matchAlphaToAlpha(alpha * depthAlpha, alpha) : alpha * depthAlpha * smoothAlpha;

    return alpha;
}

vec3 readColorConfig(ColorConfig config, float yOffset){
    vec3 col = bgEnviromentLight(config.picker, config.whitePer).rgb;
    col *= config.multi;
    return col;
}

vec3 readColorConfig(ColorConfig config){
    return readColorConfig(config, 0.);
}

vec3 getNormal(vec2 uv){
    uv.y /= 2.;
    vec3 col = texture(uTex, uv).rgb;
    col = col * 2. - 1.;
    return col;
}


vec4 getUTex(vec2 uv){
    uv.y /= 2.;
    uv.y += 1. / 2.;
    if(uv.y<=0.505 || uv.y>=0.995) {
        return vec4(0.);
    }
    return texture(uTex, uv).rgba;
}

void main() {
    vec2 uv = vUv.xy;
    vec4 uTexRGB = getUTex(uv);
    if (uTexRGB.b < 0.001)return;
    float depthAlpha = calcDepthAlpha(uTexRGB.r);
    if((uReverse < 0.5 && depthAlpha < 0.01) || (uReverse > 0.5 && depthAlpha > 0.99))return;
    vec4 cloudNoise = texture(uCloudNoiseTex, uv);
    uv += cloudNoise.xy * mix(0.01, 0.025, uTexRGB.g);// noise
    vec4 diffuseColor = vec4(vec3(1.), opacity);
    diffuseColor.a *= uTexRGB.b;
    // edge
    float edge = smoothstep(uEndEdge, uStartEdge, uTexRGB.r);
    diffuseColor.a = max(diffuseColor.a * 0.5, diffuseColor.a - edge * edge * cloudNoise.z);// dont use
    diffuseColor.a *= smoothstep(1.0, 0.5, uTexRGB.g);// 背面透明度
    if (diffuseColor.a < 0.001)return;
    highp vec3 mapN = getNormal(uv);
    mapN.xy *= normalScale;
    highp vec3 normal = normalize(mapN) * (gl_FrontFacing ? 1.0 : - 1.0);
    DirectionalLight lights[4];
    lights[0].direction = directionalLights0;
    lights[1].direction = directionalLights1;
    lights[2].direction = directionalLights2;
    lights[3].direction = directionalLights0;
    lights[3].direction.z = -lights[3].direction.z;

    float lightStrengths[4];
    lightStrengths[0] = max(dot(normalize(lights[0].direction), normal), 0.0) * ONE_OVER_PI;
    lightStrengths[1] = max(dot(normalize(lights[1].direction), normal), 0.0) * ONE_OVER_PI;
    lightStrengths[2] = max(dot(normalize(lights[2].direction), normal), 0.0) * ONE_OVER_PI;
    lightStrengths[3] = max(dot(normalize(lights[3].direction), normal), 0.0) * ONE_OVER_PI;

    /**
    * Color Config
    */
    ColorConfig bgBaseConfig = configs[0];
    ColorConfig bgDiffuseConfig = configs[1];
    ColorConfig bgSunConfig = configs[2];
    ColorConfig bgGroundConfig = configs[3];
    ColorConfig bgAmbientConfig = configs[4];
    ColorConfig bgBackLightConfig = configs[2];

    bgSunConfig.picker = mix(configs[2].picker, configs[3].picker, lightStrengths[0]);
    bgGroundConfig.picker = mix(configs[3].picker, configs[4].picker, lightStrengths[1]);
    bgAmbientConfig.picker = mix(configs[4].picker, 1.0, lightStrengths[2]);
    bgBackLightConfig.picker = mix(configs[2].picker, configs[3].picker, lightStrengths[3]);

    bgSunConfig.whitePer += mix(0.05, -0.15, lightStrengths[0]);
    bgGroundConfig.whitePer += mix(0.05, -0.15, lightStrengths[1]);
    bgAmbientConfig.whitePer += mix(0.05, -0.15, lightStrengths[2]);
    bgBackLightConfig.whitePer += mix(0.05, -0.15, lightStrengths[3]);

    float backMulti = smoothstep(0.05, 1.0, mix(1. - cloudDepth, 1., uTexRGB.g));
    float baseEdgeMulti = mix(1., 2.1, backMulti * min(1., bgSunConfig.intensity));
    baseEdgeMulti = mix(baseEdgeMulti, 1., whitePer * 0.9);

    bgBaseConfig.multi *= mix(1., 1.5, backMulti);// 越薄的地方，亮度越高
    bgBaseConfig.multi *= baseEdgeMulti;
    bgBackLightConfig.picker = bgBackLightConfig.picker;
    bgBackLightConfig.multi = mix(0., 1.3, backMulti);// 太阳照到边缘薄的地方，增加亮度
    bgBaseConfig.picker = mix(0.05, bgBaseConfig.picker, cloudDepth);
    bgBaseConfig.multi = mix(1., bgBaseConfig.multi, cloudDepth);
    bgBaseConfig.whitePer = mix(0., bgBaseConfig.whitePer, cloudDepth);

    /**
    * Read Color Config
    */
    vec3 bgBaseColor = readColorConfig(bgBaseConfig);
    vec3 bgDiffuseColor = readColorConfig(bgDiffuseConfig);
    vec3 bgSunColor = readColorConfig(bgSunConfig);
    vec3 bgGroundColor = readColorConfig(bgGroundConfig);
    vec3 bgAmbientColor = readColorConfig(bgAmbientConfig);
    vec3 bgBackLightColor = readColorConfig(bgBackLightConfig);
    /*
    * diy noon color
    */
    bgSunColor = mix(bgSunColor, vec3(1., 0.74, 0.53), whitePer);
    bgGroundColor = mix(bgGroundColor, vec3(0.85, 0.54, 0.37), whitePer);
    bgAmbientColor = mix(bgAmbientColor, vec3(0.96, 0.63, 0.42), whitePer);
    bgBackLightColor = mix(bgBackLightColor, vec3(1., 0.74, 0.53), whitePer);

    /**
    * Prepare Diffuse Color
    */
    bgDiffuseColor = mix(vec3(uTexRGB.r), vec3(1.), 0.8);
    bgDiffuseColor = mix(bgDiffuseColor, vec3(0.8), uFogDensity);
    diffuseColor.rgb = bgDiffuseColor;

    /*
    * Prepare Lights
    */
    //lights[0].intensity = (isBehindSun == 1) ? 0.3 : bgSunConfig.intensity;
    //lights[1].intensity = (isBehindSun == 1) ? 0.3 :  bgGroundConfig.intensity;
    //lights[2].intensity = (isBehindSun == 1) ? 0.3 : bgAmbientConfig.intensity;
    //lights[3].intensity = ((isBehindSun == 1) ? 1.5 : 1.0) * bgBackLightConfig.intensity;
    lights[0].intensity = mix(0.3, bgSunConfig.intensity, uSunIntensity);
    lights[1].intensity = mix(0.3, bgGroundConfig.intensity, uSunIntensity);
    lights[2].intensity = mix(0.3, bgAmbientConfig.intensity, uSunIntensity);
    lights[3].intensity = mix(1.5, 1.0, uSunIntensity) * bgBackLightConfig.intensity;
    /**
    * Cloud Color
    */
    vec3 _r = bgBaseColor;
    float fogMulti = (1. - pow(max(0., uFogDensity), 2.));
    _r += lightStrengths[0] * bgSunColor * lights[0].intensity * fogMulti;
    _r += lightStrengths[1] * bgGroundColor * lights[1].intensity * fogMulti;
    _r += lightStrengths[2] * bgAmbientColor * lights[2].intensity * fogMulti;
    _r += lightStrengths[3] * bgBackLightColor * lights[3].intensity * fogMulti;
    vec3 lightningColor = mix(bgBaseColor, vec3(1.), 0.1);
    _r += lightningColor * (0.1 + pow(smoothstep(0.05, 1., uTexRGB.a), 1.)) * uInnerIntensity * 0.4;
    _r *= mix(1., 1.5, uInnerIntensity);
    _r *= diffuseColor.rgb;
    _r = mix(_r, bgEnviromentLight(0.95).rgb, uFogDensity);
    fragColor = vec4(_r, diffuseColor.a);

    /*
    * prepare for cloud through text
    */
    float cuttingAlpha = depthCuttingAlpha(fragColor.a, depthAlpha);
    fragColor.a = max(0., min(1., cuttingAlpha));
    /*    if(fragColor.a > 0.99){
            fragColor = vec4(1.0, 0.0, 0.0, 1.0);
        }*/
}