#version 310 es
precision lowp float;
in vec4 vUv;
in vec2 vCenter;

uniform float alpha;

out vec4 oFragColor;


layout(std140) uniform TimeParam {
    float time;
    float deltaTime;
};

#define TwinkleInterval 40.
#define PI 3.1415926
#define PI2 6.2831852

float drawStar(vec2 uv, float br, float size, bool showGlow) {
    vec2 center = vec2(0.,0.);
    vec2 offset = uv - center; //当前像素相对于圆心的偏移量
    float d = dot(offset, offset);
    if(d>1.0) {
        return 0.;
    }
    float sqrtd = 1. - clamp(sqrt(d), 0., 1.);
    float starColor = br * (smoothstep((1. - size), 1., sqrtd)); //基础的星星
    starColor *= (0.8 + sin(vCenter.y * 10. + vCenter.x * 71.9 + time * 2.) * 0.4);
    if(showGlow) {
        float totalGlow = 0.;
        //绘制大范围的光晕
        float bigGlowColor = br * (smoothstep(0., 1., sqrtd));
        bigGlowColor = pow(clamp(bigGlowColor, 0., 2.), 2.) * 0.04;
        totalGlow = bigGlowColor;//发光汇总
        //绘制4角闪光
        float angle = atan(offset.y, offset.x) + PI * 0.25;//当前像素点相对于圆心的角度
        float angleOffset = abs(mod(angle, PI * 0.5) - (PI * 0.25));//当前像素点角度相对于±45、±135度的偏移
        float length = mix(0.65, 0.8, pow(angleOffset, 0.5) / (PI * 0.25));//计算这个角度散发的光线长度
        float rayGlowColor = (smoothstep(length+0.02, 1., sqrtd)) * 0.6;
        totalGlow += rayGlowColor;

        if (sqrtd > 0.8) {
            float smallGlowColor = br * (smoothstep(0.8, 1., sqrtd));
            smallGlowColor = pow(clamp(smallGlowColor, 0., 2.), 2.) * 0.33333333;
            totalGlow += smallGlowColor;
        }

        float t = vCenter.y * 721.3 + vCenter.x * 37.1+time * 2.;
        float tWait = (mod(t, (2. + TwinkleInterval) * PI) <= PI2) ? mod(t, PI2) : PI2;
        tWait -= PI * 0.5;
        totalGlow *= (0.5 + sin(tWait) * 0.5);

        starColor += totalGlow*vUv.w;
    }
    return starColor;
}

void main() {
    vec2 uv = vUv.xy*2. - 1.;
    float brScale = mix(0.6, 2., vUv.z);
    float br = 2.*brScale;

    float starColor = clamp(drawStar(uv, br, 0.1, brScale>0.8),0.,1.);

    oFragColor = vec4(vec3(1.), vUv.w*alpha*starColor);
}