#version 300 es
precision highp float;
in vec2 vUv;

uniform vec2 uResolution;
uniform float uOpacity;
uniform float uAlpha;
uniform float uTime;
uniform float uHeaviness;
uniform float uFilter;
uniform sampler2D uNoiseTex;
uniform float uQuality;
out vec4 oFragColor;

#define WIDTH 0.3
#define SIZE 1.0
#define SPEED_Y 1.4
#define NOISE_SIZE 256.
#define CELL_NOISE_COORD 0.005859375 // = 1.5/NOISE_SIZE
#define CLOSEST_INDEX 2.1 //最近可显示的图层编号，原版5
#define FURTHEST_INDEX 4.1//最远可显示的图层编号, 原版24

/* 雪的强度映射，0.是小雪，0.5是中雪，1.是大雪 */
float fixRamp(float a, float b, float c, float y) {
    float r = mix(a, b, smoothstep(0.0, 1.0, clamp(y, 0.0, 0.5) * 2.0));
    r = mix(r, c, smoothstep(0.0, 1.0, (clamp(y, 0.5, 1.0) - 0.5) * 2.0));
    return r;
}

float N11(float n) {
    return fract(sin(n * 871.213) * 3134.422);
}

float N21(vec2 uv) {
    return N11(N11(uv.x) + uv.y);
}

/* 去除随机 */
vec4 weatherEffect(vec2 uv) {

    float time = uTime + 100.; //加100为了避免时间倒流

    float layerDensity = fixRamp(1.3, 1.9, 2.6, uHeaviness); //每一层的雪花密度
    float depth = fixRamp(4.8, 4.2, 3.8, uHeaviness); //每一层之间的距离
    float preDepth = depth * 0.03; //预乘，优化for循环性能

    vec2 st = uv * 0.5; //缩放后的uv坐标
    st.x *= uResolution.x / uResolution.y;

    vec2 scaledUV = st * depth * layerDensity; //预乘，用于减少for循环内的计算量
    float preYMove = SPEED_Y * time * layerDensity; //预乘，优化for循环性能


    vec3 outcome = vec3(0.0); //其r通道用于最后输出alpha
    float dof = 5. * sin(time * .1); //影响雪花大小的随机程度
    dof = 5.;

    //循环绘制这么多层的雪 图层
    //每个0~1的uv小格内，只会有一个雪花
    float q = uQuality * 3.0;
    for (float layerIndex = CLOSEST_INDEX; layerIndex < FURTHEST_INDEX + q; layerIndex++) {

        if(uFilter >= 0.) {
            if(layerIndex > abs(uFilter)) continue;
        } else {
            if(layerIndex <= abs(uFilter)) continue;
        }

        vec2 depUV = layerIndex * scaledUV; //放大uv，缩小视图。uLayerDensity通过缩放uv，来调整密度
        //float randNum1 = texture(uNoiseTex, vec2(layerIndex * CELL_NOISE_COORD)).r; //尝试过噪声图，优化效果不理想
        depUV += vec2(depUV.y * WIDTH * (fract(layerIndex * 7.238917) - 0.5),    preYMove / (1. + layerIndex * preDepth)); //x轴随着depUV的y左右偏移，y轴随着时间下移

        vec2 cellIndex = floor(depUV);

        vec3 randNum2 = texture(uNoiseTex, vec2((cellIndex + layerIndex) * CELL_NOISE_COORD)).xyz; //每个小格子从噪声图生成一个0-1的随机数
        //优化点，是否舍弃雪花的形状
        vec2 posOffset = vec2((sin(N21(cellIndex) * 128. + time) * .4), (sin(N11(N21(cellIndex)) * 128. + time) * .4)); //圆心相对于方格中心的偏移量
        vec2 posUV = abs(fract(depUV + posOffset) - 0.5/*  - 0.95 + 0.9 * randNum2.xy */); //随机移动每个格子内坐标系的原点位置，同时xy轴对称为正数
        posUV += 0.01 * abs(2. * fract(10. * depUV.yx) - 1.); //每个格子内生成规律噪波，每个小格内循环1-0-1，10次，使雪花形状更多变

        float d = .6 * max(posUV.x - posUV.y, posUV.x + posUV.y) + max(posUV.x, posUV.y) - .01; // //在原点周围生成八边形图案，从 0 → 1.5左右；计算的性能尚可
        float edge = .005 + .05 * min(.5 * abs(layerIndex - 5. - dof), 1.); //控制雪花大小，根据图层编号和随机变量，生成edge；计算的性能尚可
        float snowflake = smoothstep(edge * SIZE * layerDensity, -edge * SIZE * layerDensity, d) * (randNum2.x / (1. + 0.02 * layerIndex * depth));

        outcome += vec3(snowflake); //添加雪花
    }
    return vec4(1., 1., 1., outcome.r * uAlpha);
}

void main() {
    vec2 uv = vUv;

    vec4 effect = weatherEffect(uv);
    effect.a *= uOpacity;

    oFragColor = effect;
}
