#version 300 es
precision highp float;
in vec2 vUv;
flat in float vtargetAlpha[5];
flat in float vtargetScale[5];
flat in float uIntensity1;
flat in float uIntensity2;
flat in float uIntensity3;
flat in float uIntensity4;
flat in int uFilter1;

uniform float uOpacity;
uniform float uAlpha;
uniform float uTime;
uniform float uSpeedY;
uniform vec2 uResolution;
uniform sampler2D uNoiseTex;
uniform int uQuality;
out vec4 oFragColor;

float PI = 3.14;
#define NOISE_SIZE 256. //噪声图的大小
#define CELL_NOISE_COORD 0.005859375 // = 1.5/NOISE_SIZE
#define SCALE_X 6.6
#define SCALE_Y 0.028
#define SCALE 5.

/* 获取随机值 */
vec3 getRandomLite (float value, int seed) {
    vec3 rg = texture(uNoiseTex, vec2((value + float(seed)) * CELL_NOISE_COORD)).rgb;
    return rg;
}

vec4 drawDropLayer(vec2 uv, float scale, float alpha, int index) {
    uv *= scale;
    float time = uTime;
    vec2 coord = uv * uResolution.xy;
    uv = (coord.xy - 0.5 * uResolution.xy) / uResolution.y; //创建坐标系，y轴从下到上是-0.5～0.5，x轴是从左到右理论上是-0.5～0.5，但是会跟随画面被等比裁切。新坐标系的原点在屏幕中央。
    vec3 color = vec3(1.0);   // 雨滴的颜色
    uv.x = uv.x * SCALE_X;    // 放大x轴坐标，乘的数字即雨滴的轨道数量
    float dx = fract(uv.x);   // 给x轴坐标取小数，生成竖向阶梯
    uv.x = floor(uv.x);       //每个竖轨道内的x坐标是固定的
    float dropTime = (time + 100.) * uSpeedY; //时间
    uv.y *= uIntensity2; //缩小y坐标
    vec3 rg = getRandomLite(uv.x, index); //获取随机值，这里没有用sin方案，那个在雨小时规律性有点强
    float offset = (rg.x + rg.y - 1.5) * 3.; //每个竖条内获得统一的-1～1的随机值
    float speed = offset * 0.033 + 0.7; //每个竖条内获得统一的随机值，0.6 - 0.8

    float trail = 95.; //每个竖条根据速度，计算一个统一的值
    float yv = fract(uv.y + dropTime * speed - 0.1 * uSpeedY * uIntensity3 + offset) * trail; //根据竖条内的y坐标和一些随机值，计算在y轴上变化的yv
    yv = 1.0 / yv; //生成向下的条带
    yv = smoothstep(0.0, 1.0, yv * yv); //归一化条带的值
    float alphaGradient = clamp(yv + 0.5, 0., 1.);
    yv = sin(yv * PI) * (speed * 5.0);
    float d2 = sin(dx * PI);
    yv *= d2 * d2;//生成圆形头
    if (index == 0 && rg.b > 0.4 && uIntensity1 > 4.5) {
        return vec4(color, smoothstep(0., 1., yv) * alphaGradient * alpha);
    } else if (index != 0 && offset > uIntensity4) {
        return vec4(color, smoothstep(0., 1., yv) * alphaGradient * alpha);
    }
    return vec4(0.);
}
vec4 weatherEffect(vec2 uv) {
    //开始绘制
    vec4 color = vec4(0.);

    //绘制最近的一层，特殊处理
    float targetAlpha1 = 0.8;
    float targetScale1 = 1.;
    if (uFilter1 >= 0){
        color += drawDropLayer(uv, 10. + targetScale1 * SCALE, uAlpha * targetAlpha1, 0);
        color = clamp(color, 0., 1.);
        color.a *= mix(1., 0.7, smoothstep(0.1, 1., uv.y));
    }
    int q = uQuality * 3;
    for (int i = 0; i < 2 + q; i++) {
        if (vtargetScale[i] == 0.0) {
            continue;
        }
        color += drawDropLayer(uv, 10. + vtargetScale[i] * SCALE, uAlpha * vtargetAlpha[i], i+1);
        color = clamp(color, 0., 1.);
        color.a *= mix(1., 0.5, smoothstep(0.1, 1., uv.y));
    }
    return color;
}

void main() {
    vec2 uv = vUv;
    vec4 col = weatherEffect(uv);
    col.a *= uOpacity;
    oFragColor = col;
}
