#version 300 es
precision lowp float;

in vec2 vUv;
out vec4 fragColor;

uniform sampler2D uTex;
uniform float opacity;
uniform sampler2D uCloudNoiseTex;
uniform float uStartEdge;
uniform float uEndEdge;
uniform float uSmooth;
uniform float uProgress;
uniform float uReverse;

float perFromVal(float val, float from, float to) {
    return (val - from) / (to - from);
}

float matchAlphaToAlpha(float curAlpha, float tarAlpha) {
    return (tarAlpha - curAlpha) / (1. - curAlpha);
}

float depthCuttingAlpha(float alpha, float depth){
    float per = (1. + uSmooth) - clamp(uProgress, 0., 1.);
    float depthAlpha = clamp(perFromVal(depth, per - uSmooth, per), 0., 1.);
    float smoothAlpha = clamp(uProgress, 0., 2.);
    smoothAlpha = 1.- clamp(smoothAlpha-1., 0.,1.);
    smoothAlpha *= clamp(uProgress*3., 0., 1.);
    bool reverse = uReverse == 1.;
    alpha *= reverse ? matchAlphaToAlpha(alpha * depthAlpha, alpha) : alpha * depthAlpha * smoothAlpha;

    return alpha;
}

float calcDepthAlpha(float depth){
    float per = (1. + uSmooth) - clamp(uProgress, 0., 1.);
    return clamp(perFromVal(depth, per - uSmooth, per), 0., 1.);
}

vec4 getUTex(vec2 uv){
    uv.y /= 2.;
    uv.y += 1. / 2.;
    if(uv.y<0.505 || uv.y>0.995) {
        return vec4(0.);
    }
    return texture(uTex, uv).rgba;
}

void main() {
    vec4 uTexRGB = getUTex(vUv);
    //if(uReverse < 0.5 && calcDepthAlpha(uTexRGB.r) < 0.001) {
    //    discard;
    //}
    float depth = uTexRGB.r;//getDep(vUv);
    // lambert diffuse
    vec4 diffuseColor = vec4(vec3(1.), opacity);
    diffuseColor.a *= uTexRGB.b;//getAlpha(uv);
    if(diffuseColor.a < 0.001) {
        discard;
    }
    // edge
    vec4 cloudNoise = texture(uCloudNoiseTex, vUv);
    float edge = smoothstep(uEndEdge, uStartEdge, depth);
    diffuseColor.a = max(diffuseColor.a * 0.5, diffuseColor.a - edge * edge * cloudNoise.z);// dont use
    diffuseColor.a *= smoothstep(1.0, 0.5, uTexRGB.g);// 背面透明度
    if(diffuseColor.a < 0.001) {
        discard;
    }
    fragColor = diffuseColor;

    /*
    * prepare for cloud through text
    */
    float cuttingAlpha = depthCuttingAlpha(fragColor.a, depth);
    fragColor.a = max(0., min(1., cuttingAlpha));
    //if(uReverse < 0.5 && fragColor.a<0.999) {
    //    discard;
    //}
    if(fragColor.a<0.95) {
        discard;
    }
}