#version 300 es
precision lowp float;
in vec2 vUv;
out vec4 oFragColor;

//uniform vec2 uResolution;
uniform float uOpacity;

uniform float uAlpha;
uniform float uLineAlpha;
uniform float uAnnulusAlpha;
uniform float uCircleAlpha;
uniform float uCircleOffset;
uniform float uCircleOffsetRatio;

uniform sampler2D uNoiseTex;
uniform sampler2D uAnnulusTex;
uniform sampler2D uObviousLineTex;
uniform sampler2D uSunTex;
uniform float uQuality;
uniform float u22Open;
uniform bool uCloseCircle;

#define uR 1.25
#define uG 1.61
#define uB 1.84

in vec4 light;
in float circleNumber;
in float randSeed;
in vec2 sunPos;
in mat4 worldSpaceMat;
//没用的uniform写在顶部的声明上，方便后期微调效果

#define SUN_DEGREE 2.
#define LINE_SPEED 0.5
#define LINE_COLOR vec4(1.0, 0.8078, 0.3922, 1.0)

#define CIRCLE_SIZE 0.95

#define DECAY 6. //光衰减，总体控制
#define PI 3.1415926
#define NOISE_SIZE 256.
#define CELL_NOISE_COORD 0.005859375

vec3 getRandomLite(float value, float seed) {
    vec3 rg = texture(uNoiseTex, vec2((value + seed) * CELL_NOISE_COORD)).rgb;
    return rg;
}

float getRandom(float value, float seed) {
    return fract(sin(value * 999.9) * seed);
}

const vec3 colorLUT[] = vec3[](vec3(0.1686, 0.7412, 0.2353), //0
vec3(1.0, 0.3843, 0.0), //1
vec3(1.0, 0.5333, 0.0), //2
vec3(0.7804, 1.0, 0.3725), //3
vec3(1.0, 0.5333, 0.0), //4
vec3(1.0, 0.3843, 0.0), //5
vec3(1.0, 0.5333, 0.0), //6
vec3(1.0, 0.5333, 0.0), //7
vec3(1.0, 0.2588, 0.1294));//8

/*获得每个圆形的颜色*/
vec3 getColor(int index) {
    return colorLUT[index];
}

vec4 circleTex(sampler2D tex, vec2 st){
    return texture(tex, vec2(length(st * 2.0 - 1.0), 0.5));
}

#define ORB_FLARE_COUNT	8
#define DISTORTION_BARREL 1.3

vec2 GetDistOffset(vec2 uv, vec2 pxoffset) {
    vec2 tocenter = uv.xy;
    vec3 prep = normalize(vec3(tocenter.y, -tocenter.x, 0.0));

    float angle = length(tocenter.xy) * 2.221 * DISTORTION_BARREL;
    vec3 oldoffset = vec3(pxoffset, 0.);

    vec3 rotated = oldoffset * cos(angle) + cross(prep, oldoffset) * sin(angle) + prep * dot(prep, oldoffset) * (1. - cos(angle));

    return rotated.xy;
}

vec3 flare(vec2 uv, vec2 pos, float dist, float size) {
    float r = max(.01 - pow(length(uv + (dist - .05)*pos), 2.4) *(1. / (size * 2.)), 0.) * 6.0;
    float g = max(.01 - pow(length(uv +  dist       *pos), 2.4) *(1. / (size * 2.)), 0.) * 6.0;
    float b = max(.01 - pow(length(uv + (dist + .05)*pos), 2.4) *(1. / (size * 2.)), 0.) * 6.0;
    return vec3(r, g, b);
}

vec3 ring(vec2 uv, vec2 pos, float dist) {
    vec2 uvd = uv*(length(uv));
    float r = max(2. / (1. + 32. * pow(length(uvd + (dist - .05) * pos), 2.)), 0.) * .25;
    float g = max(2. / (1. + 32. * pow(length(uvd +  dist * pos), 2.)), 0.) * .23;
    float b = max(2. / (1. + 32. * pow(length(uvd + (dist + .05) * pos), 2.)), 0.) * .21;
    return vec3(r, g, b);
}
vec3 lensflare(vec2 uv, vec2 pos, float brightness, float size) {
/*    vec2 posOffset = GetDistOffset(uv, pos);
    vec3 c = flare(uv, posOffset, -1., size) * 3.;
    c += flare(uv, posOffset, .5, .8 * size) * 2.;
    c += flare(uv, posOffset, -.4, .8 * size);*/
    vec3 c = ring(uv, pos, -1.) * .5 * size;
    c += ring(uv, pos, 1.) * .5 * size;

    return c * brightness;
}

vec3 lensflareSimple(vec2 uv, vec2 pos, float brightness, float size) {
    vec2 uvd = uv*(length(uv));
    float g = max(2. / (1. + 32. * pow(length(uvd - pos), 2.)) + 2. / (1. + 32. * pow(length(uvd + pos), 2.)), 0.);
    return vec3(g, g, g) * .115 * size * brightness;
}

vec3 getSun(vec2 sunPos) {
    float sunHeight = (max(0., sunPos.y * .75 + .25));
    return vec3(sunPos, sunHeight);
}


float perFromVal(float val, float from, float to) {
    return (val - from) / (to - from);
}

/* 绘制圆形光斑 */
vec3 getCircle(int index, vec2 uv, float size, float dist, vec2 sunPos, float blur) {

    blur = mix(0.45, 1., blur);
    vec3 color = getColor(index);
    vec3 f = vec3(0.);

    float c = max(0.01 - pow(length(uv + sunPos * dist), size * 1.4), 0.) * 50.;
    float c1 = (c > 0.1) ? (0.1 - (c - 0.1) / 5.) : (c / 1.4) * blur * 2.;
    float c2 = mix(c1, c, (blur > 0.5) ? ((blur - 0.5) * 2.) : 0.);

    float alpha = uCircleAlpha * mix(1., 0.4, (size - 1.) * 3.) * ((blur >= 0.82) ? 0.5 : 1.);
    f += c2 * color * alpha;

    return f - 0.01;
}

vec4 getWeatherEffect(vec2 uv) {

    float number = uCloseCircle ? 0.0 : circleNumber;
    //开始绘制
    vec3 color = vec3(0.);
    float lsun = length(sunPos);

    if (uQuality > 0.) {
        float d = abs(sunPos.x * uv.y - sunPos.y * uv.x) / lsun;
        if (d > 0.1) {
            color -= 0.01 * number;
            if(u22Open > 0.5){
                color += lensflareSimple(uv * 4.0, sunPos.xy * 1.5, 0.6, 4.0) * vec3(1.4, 1.2, 1.);
            }
        } else {
            float l = dot(normalize(sunPos), uv);
            float threshL = length(clamp(sunPos * uCircleOffset, -1.0, 0.0));
            if (l > threshL + 0.1 || l < -0.8 * lsun + threshL - 0.1) {
                color -= 0.01 * number;
                if(u22Open > 0.5){
                    color += lensflareSimple(uv * 4.0, sunPos.xy * 1.5, 0.6, 4.0) * vec3(1.4, 1.2, 1.);
                }
            } else {
                for (float i = 0.; i < number; i++) {
                    //使用噪声图实现
                    vec3 rg = getRandomLite(i, randSeed);
                    float staticSize = rg.r * 0.33 + CIRCLE_SIZE;
                    float size = staticSize + light.a;
                    float staticDist = uCircleOffset + rg.g * mix(0.5, 0.2, uCircleOffsetRatio) + (i / 30. + pow(i, 2.) * 0.01);
                    float dist = staticDist + light.a;
                    color += getCircle(int(i), uv, size, dist, sunPos, rg.b);
                }
                if(u22Open > 0.5){
                    color += lensflare(uv * 4.0, sunPos.xy * 1.5, 0.6, 4.0) * vec3(1.4, 1.2, 1.);
                }
            }
        }
    }

    // √ 绘制一个环形光晕
    vec2 diff = uv - sunPos;
    vec2 st = uv * 3.0; // 中心坐标系
    st -= sunPos * 2.5; // 太阳位置


    if(u22Open > 0.5){
        vec4 clipSpacePos = worldSpaceMat * vec4(st, 0.0, 1.0); // invert
        vec2 ndcSpacePos = clipSpacePos.xy / clipSpacePos.w + 0.5;
        color += circleTex(uAnnulusTex, ndcSpacePos.xy).rgb * uAnnulusAlpha * 4.0;
    }

    //射线，重点优化
    // √ 静态部分
    vec2 st2 = clamp((diff) / 3.0 + 0.5, 0.0, 1.0);
    vec3 obviousLine = texture(uObviousLineTex, st2).rgb;
    float fan = min(max(0.3, pow(perFromVal(dot(normalize(sunPos), normalize(sunPos - uv)), -1.0, 1.0), 3.0)), 0.8);
    fan = max( pow(1.0 - length(sunPos - uv), 3.0), fan );
    color += obviousLine * fan;
    //动态部分
    float angle = atan(diff.y, diff.x);
    vec3 angleArr = vec3(angle, angle + PI / 5., angle + PI / 2.);
    vec3 dimLine = abs(sin(angleArr * 3. + cos(angleArr * 9.))) * (abs(sin(angleArr * 9.))) * light.rgb;
    color += vec3(dimLine.x + dimLine.y + dimLine.z) * 1.2 * (1.0 - st2.x);

    vec2 st3 = clamp(diff + 0.5, 0.0, 1.0);
    color += circleTex(uSunTex, st3).rgb  * vec3(uR, uG, uB) * 1.02;

    vec2 st4 = vec2(length(st3 * 2.0 - 1.0) * 2.8, 0.5);
    // plus
    // vec4 glow = vec4(0.4, 0.314, 0.184, mix(1.0, 0.0, st4.x));
    vec4 glow = vec4(0.8509803922, 0.6039215686, 0.3490196078, 0.5 / exp(st4.x * st4.x));
    color.rgb += glow.rgb * glow.a;

    color *= exp(1.0 - length(diff)) / DECAY;
    color = clamp(color, 0.0, 1.0) * smoothstep(-0.5, 0.1, uv.y);

    return vec4(color, 1.);
}

void main() {
    vec2 uv = vUv;
    vec4 effect = getWeatherEffect(uv);
    effect.a *= uOpacity;
    oFragColor = effect;
}