#version 300 es
precision highp float;

in vec2 vUv;
out vec4 fragColor;

uniform float uOpacity;
uniform float uShineTime;
uniform bool uShine;
uniform float uStaticTime;
uniform vec2 uResolution;

//uniform sampler2D uNoiseTex;
//uniform sampler2D uPerlinTex;

//uniform sampler2D uDebugGridTex;
uniform sampler2D uStarMapTex;

// #define uGalaxyAlpha 0.16
// #define uGalaxyTheta 0.67
// #define uGalaxyPhi 0.00
// #define uGalaxyFov 0.27

uniform float uGalaxyAlpha;
uniform float uGalaxyTheta;
uniform float uGalaxyPhi;
uniform float uGalaxyFov;

#define progress uOpacity
#define PI 3.141592653
#define TWOPI (2.0 * PI)

float N(float t) {
    return fract(sin(t*12345.564)*7658.76);
}

vec3 N13(float p) {
    vec3 p3 = fract(vec3(p) * vec3(.1031,.11369,.13787));
    p3 += dot(p3, p3.yzx + 19.19);
    return fract(vec3((p3.x + p3.y)*p3.z, (p3.x+p3.z)*p3.y, (p3.y+p3.z)*p3.x));
}

float atan2(in float y, in float x) {
    bool s = (abs(x) > abs(y));
    return mix(PI/2.0 - atan(x,y), atan(y,x), s);
}

float luminance( const in vec3 rgb ) {
	const vec3 weights = vec3( 0.2126729, 0.7151522, 0.0721750 );
	return dot( weights, rgb );
}

float perFromVal(float val, float min, float max) {
    return (val - min) / (max - min);
}

float saw(float b, float t) {
    return smoothstep(0., b, t)*smoothstep(1., b, t);
}

vec3 saturation(vec3 rgb, float adjustment) {
    const vec3 W = vec3(0.2125, 0.7154, 0.0721);
    vec3 intensity = vec3(dot(rgb, W));
    return mix(intensity, rgb, adjustment);
}

float pfv(float val, float min, float max) {
  return (val - min) / (max - min);
}

float cpfv(float val, float min, float max) {
  return clamp(pfv(val, min, max), 0.0, 1.0);
}

vec4 weatherEffect(vec2 uv) {
    float fov = uGalaxyFov * 2.0 * PI;
    vec2 st = (uv - 0.5) * 2.0; // -1 -> 1
    st.x *= uResolution.x / uResolution.y;

    if (uResolution.x / uResolution.y > 0.8) {
      st /= 1.4;
    }

    if (uResolution.x / uResolution.y > 1.5) {
      st.x += uResolution.x / uResolution.y / 2.5;
      // st /= 1.5;
    }

    vec3 lookat = vec3(0.0, 0.0, -1.0);
    vec3 offset = vec3(st, 0.0);

    float rx0 = st.y * fov;
    float ry0 = st.x * fov;


    // float edgeFade = smoothstep(PI / 3.0, PI / 2.5, abs(ry0));
    float edgeFade = cpfv(abs(ry0), PI / 3.0, PI / 2.5);
    // float edgeFade = cpfv(sqrt(rx0*rx0 + 2.0*ry0*ry0), PI / 2.5, PI / 1.8);

    mat4 rotateX0 = mat4(
        1.0, 0.0, 0.0, 0.0,
        0.0, cos(rx0), -sin(rx0), 0.0,
        0.0, sin(rx0), cos(rx0), 0.0,
        0.0, 0.0, 0.0, 1.0
    );
    mat4 rotateY0 = mat4(
        cos(ry0), 0.0, sin(ry0), 0.0,
        0.0, 1.0, 0.0, 0.0,
        -sin(ry0), 0.0, cos(ry0), 0.0,
        0.0, 0.0, 0.0, 1.0
    );
    mat4 rotate0 = rotateX0 * rotateY0;

    float rx = uGalaxyAlpha * TWOPI;
    float ry = uGalaxyTheta * TWOPI;
    float rz = uGalaxyPhi * TWOPI;

    rz +=  uStaticTime / 150.0;

    mat4 rotateX = mat4(
        1.0, 0.0, 0.0, 0.0,
        0.0, cos(rx), -sin(rx), 0.0,
        0.0, sin(rx), cos(rx), 0.0,
        0.0, 0.0, 0.0, 1.0
    );
    mat4 rotateY = mat4(
        cos(ry), 0.0, sin(ry), 0.0,
        0.0, 1.0, 0.0, 0.0,
        -sin(ry), 0.0, cos(ry), 0.0,
        0.0, 0.0, 0.0, 1.0
    );
    mat4 rotateZ = mat4(
        cos(rz), -sin(rz), 0.0, 0.0,
        sin(rz), cos(rz), 0.0, 0.0,
        0.0, 0.0, 1.0, 0.0,
        0.0, 0.0, 0.0, 1.0
    );
    mat4 rotate = rotateZ * rotateY * rotateX;

    // vec3 dir = normalize(lookat + offset * fov);
    vec3 dir = (rotate0 * vec4(lookat, 0.0)).xyz;
    dir = (rotate * vec4(dir, 0.0)).xyz;

    vec2 invAtan = vec2(0.1591, 0.3183);

    vec2 tx = vec2(atan(dir.z, dir.x), asin(dir.y));
    tx *= invAtan; tx.x += 0.5; tx.y += 0.5;

    tx.y = 1.0 - tx.y;//starMap图片是y颠倒的，这里进行反转
    tx.x = 1.0 - tx.x;

    //vec4 debugColor = texture2D(uDebugGridTex, tx);
    vec4 color = texture(uStarMapTex, tx);
    color.rgb = saturation(color.rgb, 4.0);
    color.a *= smoothstep(0.1, 0.2, luminance(color.rgb));


    if(uShine){
        float t = uShineTime / 20.0;
        vec2 st2 = st * 60.;
        vec2 id = floor(st2);
        st2 = fract(st2) - .5;
        vec3 n = N13(id.x * 107.45 + id.y * 3543.654);
        vec2 p = (n.xy - .5) * .7;
        float d = length(st2 - p);
        float fade = saw(.025, fract(t + n.z));
        float c = smoothstep(.2, 0., d) * fract(n.z * 10.) * fade;
        float shine = c * 1.5 * smoothstep(0.9, 1.0, pow(progress, 4.0));
        color.rgb += shine * min((uShineTime / 2.0), 1.0);
    }

    // fade out
    float fb = clamp(mix(1.0, 0.0, 1.0/exp(4.0 * uv.y * uv.y)), 0.0, 1.0);
    float ft = clamp(0.0, 1.0, mix(1.0, 0.0, perFromVal(uv.y, 0.7, 1.0)));
    color.a *= ft;
    color.a *= fb;

    return mix(color, vec4(0.0), edgeFade);
    // return color;
}

void main() {
    vec2 uv = vUv;
    vec4 col = weatherEffect(uv);
    float exit = mix(0.0, 1.0, cpfv(uStaticTime, -30.0, 0.0)) * mix(1.0, 0.0, cpfv(uStaticTime, 60.0, 90.0));
    col.a *=  exit;
    fragColor = col;
}
